@@ -14,6 +14,7 @@ from unittest.mock import patch
 from .formatters import DefaultHelpFormatter, empty_help, formatter_context, get_env_var
 from .jsonnet import ActionJsonnet
 from .jsonschema import ActionJsonSchema
+from .link_arguments import ActionLink, ArgumentLinking
 from .loaders_dumpers import check_valid_dump_format, dump_using_format, get_loader_exceptions, loaders, load_value, load_value_context, yaml_load
 from .namespace import dict_to_namespace, is_meta_key, Namespace, split_key, split_key_leaf, strip_meta
 from .signatures import is_pure_dataclass, SignatureArguments
@@ -25,7 +26,6 @@ from .actions import (
     _ActionSubCommands,
     _ActionPrintConfig,
     _ActionConfigLoad,
-    _ActionLink,
     _is_branch_key,
     _find_action,
     _find_action_and_subcommand,
@@ -137,7 +137,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
         parser = self.parser if hasattr(self, 'parser') else self  # type: ignore
         if name is not None and name in parser.groups:
             raise ValueError(f'Group with name {name} already exists.')
-        group = _ArgumentGroup(parser, *args, **kwargs)
+        group = _ArgumentGroup(parser, *args, logger=parser.logger, **kwargs)
         group.parser = parser
         parser._action_groups.append(group)
         if name is not None:
@@ -151,7 +151,7 @@ class _ArgumentGroup(ActionsContainer, argparse._ArgumentGroup):
     parser: Optional['ArgumentParser'] = None
 
 
-class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
+class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser):
     """Parser for command line, yaml/jsonnet files and environment variables."""
 
     formatter_class: Type[DefaultHelpFormatter]  # type: ignore
@@ -162,7 +162,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
     def __init__(
         self,
         *args,
-        env_prefix: Optional[str] = None,
+        env_prefix: Union[bool, str] = True,
         error_handler: Optional[Callable[['ArgumentParser', str], None]] = usage_and_exit_error_handler,
         formatter_class: Type[DefaultHelpFormatter] = DefaultHelpFormatter,
         logger: Union[bool, str, dict, logging.Logger] = False,
@@ -194,7 +194,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
             default_env: Set the default value on whether to parse environment variables.
             default_meta: Set the default value on whether to include metadata in config objects.
         """
-        super().__init__(*args, formatter_class=formatter_class, **kwargs)
+        super().__init__(*args, formatter_class=formatter_class, logger=logger, **kwargs)
         if self.groups is None:
             self.groups = {}
         self.required_args: Set[str] = set()
@@ -205,7 +205,6 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         self.env_prefix = env_prefix
         self.parser_mode = parser_mode
         self.dump_header = dump_header
-        self.logger = logger
         self.error_handler = error_handler
         self._print_config = print_config
         if version is not None:
@@ -307,7 +306,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
 
         _ActionPrintConfig.print_config_if_requested(self, cfg)
 
-        _ActionLink.apply_parsing_links(self, cfg)
+        ActionLink.apply_parsing_links(self, cfg)
 
         if not skip_check and not lenient_check.get():
             with load_value_context(self.parser_mode):
@@ -604,29 +603,6 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         return self._apply_actions(cfg_dict, prev_cfg=prev_cfg)
 
 
-    def link_arguments(
-        self,
-        source: Union[str, Tuple[str, ...]],
-        target: str,
-        compute_fn: Callable = None,
-        apply_on: str = 'parse',
-    ):
-        """Makes an argument value be derived from the values of other arguments.
-
-        Refer to :ref:`argument-linking` for a detailed explanation and examples-
-
-        Args:
-            source: Key(s) from which the target value is derived.
-            target: Key to where the value is set.
-            compute_fn: Function to compute target value from source.
-            apply_on: At what point to set target value, 'parse' or 'instantiate'.
-
-        Raises:
-            ValueError: If an invalid parameter is given.
-        """
-        _ActionLink(self, source, target, compute_fn, apply_on)
-
-
     ## Methods for adding to the parser ##
 
     def add_subparsers(self, **kwargs) -> NoReturn:
@@ -693,7 +669,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
 
         cfg = deepcopy(cfg)
         cfg = strip_meta(cfg)
-        _ActionLink.strip_link_target_keys(self, cfg)
+        ActionLink.strip_link_target_keys(self, cfg)
 
         with load_value_context(self.parser_mode):
             if not skip_check:
@@ -706,7 +682,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
 
             if skip_default:
                 defaults = self.get_defaults(skip_check=True)
-                _ActionLink.strip_link_target_keys(self, defaults)
+                ActionLink.strip_link_target_keys(self, defaults)
                 self._dump_cleanup_actions(defaults, self._actions, {'skip_check': True, 'skip_none': skip_none})
                 self._dump_delete_default_entries(cfg_dict, defaults.as_dict())
 
@@ -810,7 +786,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
 
         else:
             cfg = deepcopy(cfg)
-            _ActionLink.strip_link_target_keys(self, cfg)
+            ActionLink.strip_link_target_keys(self, cfg)
 
             if not skip_check:
                 with load_value_context(self.parser_mode):
@@ -1087,11 +1063,12 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
             components.extend(groups)
 
         components.sort(key=lambda x: -len(split_key(x.dest)))  # type: ignore
-        order = _ActionLink.instantiation_order(self)
-        components = _ActionLink.reorder(order, components)
+        order = ActionLink.instantiation_order(self)
+        components = ActionLink.reorder(order, components)
 
         cfg = strip_meta(cfg)
         for component in components:
+            ActionLink.apply_instantiation_links(self, cfg, target=component.dest)
             if isinstance(component, (ActionTypeHint, _ActionConfigLoad)):
                 try:
                     value, parent, key = cfg.get_value_and_parent(component.dest)
@@ -1101,11 +1078,11 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
                     if value is not None:
                         with load_value_context(self.parser_mode):
                             parent[key] = component.instantiate_classes(value)
-                        _ActionLink.apply_instantiation_links(self, cfg, component.dest)
             else:
                 with load_value_context(self.parser_mode):
                     component.instantiate_class(component, cfg)
-                _ActionLink.apply_instantiation_links(self, cfg, component.dest)
+
+        ActionLink.apply_instantiation_links(self, cfg, order=order)
 
         subcommand, subparser = _ActionSubCommands.get_subcommand(self, cfg, fail_no_subcommand=False)
         if subcommand is not None and subparser is not None:
@@ -1390,7 +1367,7 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
 
 
     @property
-    def env_prefix(self) -> Optional[str]:
+    def env_prefix(self) -> Union[bool, str]:
         """The environment variables prefix property.
 
         :getter: Returns the current environment variables prefix.
@@ -1399,12 +1376,17 @@ class ArgumentParser(ActionsContainer, argparse.ArgumentParser):
         Raises:
             ValueError: If an invalid value is given.
         """
-        return self._env_prefix
+        return self._env_prefix  # type: ignore
 
 
     @env_prefix.setter
-    def env_prefix(self, env_prefix: Optional[str]):
-        if env_prefix is None:
+    def env_prefix(self, env_prefix: Union[bool, str]):
+        if env_prefix is False:
+            self._env_prefix = None
+        elif env_prefix in {True, None}:
+            if env_prefix is None:
+                from .deprecated import deprecation_warning, env_prefix_property_none_message
+                deprecation_warning(ArgumentParser, env_prefix_property_none_message)
             self._env_prefix = os.path.splitext(self.prog)[0]
         elif isinstance(env_prefix, str):
             self._env_prefix = env_prefix
