@@ -9,7 +9,7 @@ from .namespace import Namespace
 from .optionals import get_config_read_mode, set_config_read_mode
 from .typehints import ActionTypeHint
 from .typing import path_type, restricted_number_type, registered_types
-from .util import _issubclass, warning
+from .util import is_subclass, warning
 
 
 __all__ = [
@@ -178,7 +178,7 @@ class ActionEnum:
 
     def __init__(self, **kwargs):
         if 'enum' in kwargs:
-            if not _issubclass(kwargs['enum'], Enum):
+            if not is_subclass(kwargs['enum'], Enum):
                 raise ValueError('Expected enum to be an subclass of Enum.')
             self._type = kwargs['enum']
         else:
@@ -245,3 +245,13 @@ cli_return_parser_message = """
     The return_parser parameter was deprecated in v4.5.0 and will be removed in
     v5.0.0. Instead of this use function capture_parser.
 """
+
+logger_property_none_message = """
+    Setting the logger property to None was deprecated in v4.10.0 and will raise
+    an exception in v5.0.0. Use False instead.
+"""
+
+env_prefix_property_none_message = """
+    Setting the env_prefix property to None was deprecated in v4.11.0 and will raise
+    an exception in v5.0.0. Use True instead.
+"""
