@@ -3,6 +3,8 @@ import inspect
 import logging
 import textwrap
 from collections import defaultdict
+from contextlib import contextmanager
+from contextvars import ContextVar
 from copy import deepcopy
 from dataclasses import dataclass
 from typing import Any, Callable, List, Optional, Tuple, Type, Union
@@ -48,6 +50,10 @@ def is_method_or_property(attr) -> bool:
     return is_method(attr) or is_property(attr)
 
 
+def is_classmethod(parent, component) -> bool:
+    return parent and isinstance(inspect.getattr_static(parent, component.__name__), classmethod)
+
+
 def ast_str(node):
     return getattr(ast, 'unparse', ast.dump)(node)
 
@@ -84,16 +90,42 @@ def ast_is_dict_assign_with_value(node, value):
     return False
 
 
-def ast_is_call_with_value(node, value) -> bool:
-    if isinstance(node, ast.Call):
-        value_dump = ast.dump(value)
-        for argtype in ['args', 'keywords']:
-            for arg in getattr(node, argtype):
-                if isinstance(getattr(arg, 'value', None), ast.AST) and ast.dump(arg.value) == value_dump:
-                    return True
+def ast_is_call_with_value(node, value_dump) -> bool:
+    for argtype in ['args', 'keywords']:
+        for arg in getattr(node, argtype):
+            if isinstance(getattr(arg, 'value', None), ast.AST) and ast.dump(arg.value) == value_dump:
+                return True
     return False
 
 
+ast_constant_attr = {
+    ast.Constant: 'value',
+    # python <= 3.7:
+    ast.NameConstant: 'value',
+    ast.Num: 'n',
+    ast.Str: 's',
+}
+
+
+def ast_is_constant(node):
+    return isinstance(node, (ast.Str, ast.Num, ast.NameConstant, ast.Constant))
+
+
+def ast_get_constant_value(node):
+    assert ast_is_constant(node)
+    return getattr(node, ast_constant_attr[node.__class__])
+
+
+def ast_is_kwargs_pop_or_get(node, value_dump) -> bool:
+    return (
+        isinstance(node.func, ast.Attribute) and
+        value_dump == ast.dump(node.func.value) and
+        node.func.attr in {'pop', 'get'} and
+        len(node.args) == 2 and
+        isinstance(ast_get_constant_value(node.args[0]), str)
+    )
+
+
 def ast_is_super_call(node) -> bool:
     return (
         isinstance(node, ast.Call) and
@@ -150,7 +182,10 @@ def get_arg_kind_index(params, kind):
 
 
 def get_signature_parameters_and_indexes(component, parent, logger):
-    params = list(inspect.signature(component).parameters.values())
+    if is_classmethod(parent, component):
+        params = list(inspect.signature(component.__func__).parameters.values())
+    else:
+        params = list(inspect.signature(component).parameters.values())
     if parent:
         params = params[1:]
     args_idx = get_arg_kind_index(params, kinds.VAR_POSITIONAL)
@@ -166,7 +201,35 @@ def get_signature_parameters_and_indexes(component, parent, logger):
             component=component,
             **{a: getattr(param, a) for a in parameter_attributes},
         )
-    return params, args_idx, kwargs_idx
+    return params, args_idx, kwargs_idx, doc_params
+
+
+ast_literals = {
+    ast.dump(ast.parse(v, mode='eval').body): lambda: ast.literal_eval(v)
+    for v in ['{}', '[]']
+}
+
+
+def get_kwargs_pop_or_get_parameter(node, component, parent, doc_params, logger):
+    name = ast_get_constant_value(node.args[0])
+    if ast_is_constant(node.args[1]):
+        default = ast_get_constant_value(node.args[1])
+    else:
+        default = ast.dump(node.args[1])
+        if default in ast_literals:
+            default = ast_literals[default]()
+        else:
+            default = None
+            logger.debug(f'AST resolver: unsupported kwargs pop/get default: {ast_str(node)}')
+    return ParamData(
+        name=name,
+        annotation=inspect._empty,
+        default=default,
+        kind=inspect._ParameterKind.KEYWORD_ONLY,
+        doc=doc_params.get(name),
+        parent=parent,
+        component=component,
+    )
 
 
 def split_args_and_kwargs(params: ParamList) -> Tuple[ParamList, ParamList]:
@@ -207,6 +270,19 @@ def common_parameters(params_list: List[ParamList]) -> ParamList:
     return common
 
 
+def merge_parameters(source: Union[ParamData, ParamList], target: ParamList) -> ParamList:
+    if not isinstance(source, list):
+        source = [source]
+    source_dict = {p.name: p for p in source}
+    replace_names = set()
+    for p in target:
+        if p.name in source_dict and p.annotation is inspect._empty and source_dict[p.name] is not inspect._empty:
+            replace_names.add(p.name)
+    target = [p for p in target if p.name not in replace_names]
+    target_names = set(p.name for p in target)
+    return target + [s for s in source if s.name not in target_names]
+
+
 def has_dunder_new_method(cls, attr_name):
     classes = inspect.getmro(cls)[1:]
     return (
@@ -216,13 +292,33 @@ def has_dunder_new_method(cls, attr_name):
     )
 
 
-def get_next_mro_class_and_method(parent, method_name):
-    classes = inspect.getmro(parent)[1:]
-    for num, cls in enumerate(c for c in classes if c is not object):
+current_mro: ContextVar = ContextVar('current_mro', default=(None, None))
+
+
+@contextmanager
+def mro_context(parent):
+    token = None
+    if parent:
+        classes, idx = current_mro.get()
+        if not classes or classes[idx] is not parent:
+            classes = [c for c in inspect.getmro(parent) if c is not object]
+            token = current_mro.set((classes, 0))
+    try:
+        yield
+    finally:
+        if token:
+            current_mro.reset(token)
+
+
+def get_mro_parameters(method_name, get_parameters_fn, logger):
+    classes, idx = current_mro.get()
+    for num, cls in enumerate(classes[idx+1:], start=idx+1):
         method = getattr(cls, method_name, None)
-        if method and not any(method is getattr(c, method_name, None) for c in classes[num+1:]):
-            return cls, method
-    return None, None
+        remainder = classes[num+1:] + [object]
+        if method and not any(method is getattr(c, method_name, None) for c in remainder):
+            current_mro.set((classes, num))
+            return get_parameters_fn(cls, method, logger=logger)
+    return []
 
 
 def get_component_and_parent(
@@ -241,17 +337,21 @@ def get_component_and_parent(
         attr = inspect.getattr_static(function_or_class, method_or_property)
         if is_staticmethod(attr):
             component = getattr(function_or_class, method_or_property)
-        elif has_dunder_new_method(function_or_class, method_or_property):
-            parent = function_or_class
+            return component, parent, method_or_property
+        parent = function_or_class
+        if has_dunder_new_method(function_or_class, method_or_property):
             component = getattr(function_or_class, '__new__')
-        elif is_method_or_property(attr):
-            parent = function_or_class
-            component = attr.fget if is_property(attr) else attr
+        elif is_method(attr):
+            component = attr
+        elif is_property(attr):
+            component = attr.fget
+        elif isinstance(attr, classmethod):
+            component = getattr(function_or_class, method_or_property)
         elif attr is not object.__init__:
-            raise ValueError(f'Invalid input: class={function_or_class}, method_or_property={method_or_property}.')
+            raise ValueError(f'Invalid or unsupported input: class={function_or_class}, method_or_property={method_or_property}')
     else:
         if not callable(function_or_class):
-            raise ValueError(f'Invalid input: function={function_or_class}.')
+            raise ValueError(f'Non-callable input: function={function_or_class}')
         component = function_or_class
     return component, parent, method_or_property
 
@@ -300,17 +400,21 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
                 self.generic_visit(node)
 
     def visit_AnnAssign(self, node):
-        self.visit_Assign(node)
+        if node.value is not None:
+            self.visit_Assign(node)
 
     def visit_Call(self, node):
         for key, value in self.find_values.items():
-            if ast_is_call_with_value(node, value):
+            value_dump = ast.dump(value)
+            if ast_is_call_with_value(node, value_dump):
                 if isinstance(node.func, ast.Attribute):
                     value_dump = ast.dump(node.func.value)
                     if value_dump in self.dict_assigns:
                         self.values_found.append((key, self.dict_assigns[value_dump]))
                         continue
                 self.values_found.append((key, node))
+            elif ast_is_kwargs_pop_or_get(node, value_dump):
+                self.values_found.append((key, node))
         self.generic_visit(node)
 
     def find_values_usage(self, values):
@@ -324,19 +428,23 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
         function_or_class = method_or_property = None
         module = inspect.getmodule(self.component)
         if isinstance(node.func, ast.Name):
-            assert hasattr(module, node.func.id), f'No attribute {node.func.id!r} in {module}'
-            function_or_class = getattr(module, node.func.id)
+            if is_classmethod(self.parent, self.component) and node.func.id == self.self_name:
+                function_or_class = self.parent
+            else:
+                function_or_class = getattr(module, node.func.id)
         elif isinstance(node.func, ast.Attribute) and isinstance(node.func.value, ast.Name):
             if self.parent and ast.dump(node.func.value) == ast.dump(ast_variable_load(self.self_name)):
                 function_or_class = self.parent
                 method_or_property = node.func.attr
             else:
-                assert hasattr(module, node.func.value.id), f'No attribute {node.func.value.id!r} in {module}'
                 container = getattr(module, node.func.value.id)
-                assert hasattr(container, node.func.attr), f'No attribute {node.func.attr!r} in {node.func.value.id!r} of {module}'
-                function_or_class = getattr(container, node.func.attr)
+                if inspect.isclass(container):
+                    function_or_class = container
+                    method_or_property = node.func.attr
+                else:
+                    function_or_class = getattr(container, node.func.attr)
         if not function_or_class:
-            self.logger.debug(f'Component not supported: {ast_str(node)}')
+            self.logger.debug(f'AST resolver: not supported: {ast_str(node)}')
             return None
         return function_or_class, method_or_property
 
@@ -348,7 +456,7 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
             kwarg = ast_get_call_kwarg_with_value(node, value)
             if kwarg:
                 if kwarg.arg:
-                    self.logger.debug(f'kwargs attribute given as keyword parameter not supported: {ast_str(node)}')
+                    self.logger.debug(f'AST resolver: kwargs attribute given as keyword parameter not supported: {ast_str(node)}')
                 else:
                     get_param_args = self.get_node_component(node)
                     if get_param_args:
@@ -371,30 +479,37 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
         if not values_found:
             return args, kwargs
 
+        kwargs_value_dump = ast.dump(kwargs_value)
         for node in [v for k, v in values_found if k == kwargs_name]:
             if isinstance(node, ast.Call):
+                if ast_is_kwargs_pop_or_get(node, kwargs_value_dump):
+                    param = get_kwargs_pop_or_get_parameter(node, self.component, self.parent, self.doc_params, self.logger)
+                    kwargs = merge_parameters(param, kwargs)
+                    continue
                 kwarg = ast_get_call_kwarg_with_value(node, kwargs_value)
                 params = []
                 if kwarg.arg:
-                    self.logger.debug(f'kwargs given as keyword parameter not supported: {ast_str(node)}')
+                    self.logger.debug(f'AST resolver: kwargs given as keyword parameter not supported: {ast_str(node)}')
                 elif self.parent and ast_is_super_call(node):
                     if not ast_is_supported_super_call(node, self.parent, self.self_name):
-                        self.logger.debug(f'super with arbitrary parameters not supported: {ast_str(node)}')
+                        self.logger.debug(f'AST resolver: super with arbitrary parameters not supported: {ast_str(node)}')
                     else:
-                        next_class, next_method = get_next_mro_class_and_method(self.parent, node.func.attr)  # type: ignore
-                        if next_method:
-                            params = get_signature_parameters(next_class, next_method, logger=self.logger)
+                        params = get_mro_parameters(node.func.attr, get_signature_parameters, self.logger)  # type: ignore
                 else:
                     get_param_args = self.get_node_component(node)
                     if get_param_args:
                         params = get_signature_parameters(*get_param_args, logger=self.logger)
-                args, kwargs = split_args_and_kwargs(remove_given_parameters(node, params))
+                args, kwargs_ = split_args_and_kwargs(remove_given_parameters(node, params))
+                kwargs = merge_parameters(kwargs_, kwargs)
+                break
             elif isinstance(node, ast_assign_type):
                 self_attr = self.parent and ast_is_attr_assign(node, self.self_name)
                 if self_attr:
-                    kwargs = self.get_parameters_attr_use_in_members(self_attr)
+                    params = self.get_parameters_attr_use_in_members(self_attr)
+                    kwargs = merge_parameters(params, kwargs)
+                    break
                 else:
-                    self.logger.debug(f'Unsupported type of assign: {ast_str(node)}')
+                    self.logger.debug(f'AST resolver: unsupported type of assign: {ast_str(node)}')
 
         return args, kwargs
 
@@ -410,7 +525,7 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
             kwargs = visitor.get_parameters_call_attr(attr_name, attr_value)
             if kwargs is not None:
                 return kwargs
-        self.logger.debug(f'Did not find use of {self.self_name}.{attr_name} in members of {self.parent}')
+        self.logger.debug(f'AST resolver: did not find use of {self.self_name}.{attr_name} in members of {self.parent}')
         return []
 
     def get_parameters_call_attr(self, attr_name: str, attr_value: ast.AST) -> Optional[ParamList]:
@@ -428,9 +543,11 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
     def get_parameters(self) -> ParamList:
         if self.component is None:
             return []
-        params, args_idx, kwargs_idx = get_signature_parameters_and_indexes(self.component, self.parent, self.logger)
+        params, args_idx, kwargs_idx, doc_params = get_signature_parameters_and_indexes(self.component, self.parent, self.logger)
         if args_idx >= 0 or kwargs_idx >= 0:
-            args, kwargs = self.get_parameters_args_and_kwargs()
+            self.doc_params = doc_params
+            with mro_context(self.parent):
+                args, kwargs = self.get_parameters_args_and_kwargs()
             params = replace_args_and_kwargs(params, args, kwargs)
         return params
 
@@ -441,12 +558,12 @@ def get_parameters_by_assumptions(
     logger: Union[bool, str, dict, logging.Logger] = True,
 ) -> ParamList:
     component, parent, method_name = get_component_and_parent(function_or_class, method_name)
-    params, args_idx, kwargs_idx = get_signature_parameters_and_indexes(component, parent, logger)
+    params, args_idx, kwargs_idx, _ = get_signature_parameters_and_indexes(component, parent, logger)
 
     if parent and (args_idx >= 0 or kwargs_idx >= 0):
-        next_class, next_method = get_next_mro_class_and_method(parent, method_name)
-        if next_method:
-            subparams = get_parameters_by_assumptions(next_class, next_method.__name__, logger)
+        with mro_context(parent):
+            subparams = get_mro_parameters(method_name, get_parameters_by_assumptions, logger)
+        if subparams:
             args, kwargs = split_args_and_kwargs(subparams)
             params = replace_args_and_kwargs(params, args, kwargs)
 
@@ -476,9 +593,10 @@ def get_signature_parameters(
     try:
         visitor = ParametersVisitor(function_or_class, method_or_property, logger=logger)
         return visitor.get_parameters()
-    except SourceNotAvailable as ex:
+    except Exception as ex:
+        cause = 'Source not available' if isinstance(ex, SourceNotAvailable) else 'Problems with AST resolving'
         logger.debug(
-            'Source not available, falling back to parameters by assumptions: '
-            f'function_or_class={function_or_class} method_or_property={method_or_property} :: {ex}'
+            f'{cause}, falling back to parameters by assumptions: function_or_class={function_or_class} '
+            f'method_or_property={method_or_property} :: {ex}'
         )
         return get_parameters_by_assumptions(function_or_class, method_or_property, logger)
