@@ -7,7 +7,6 @@ import re
 import stat
 import sys
 import warnings
-from collections import defaultdict
 from contextlib import contextmanager
 from contextvars import ContextVar
 from functools import wraps
@@ -294,38 +293,6 @@ def known_to_fsspec(path: str) -> bool:
     return False
 
 
-class DirectedGraph:
-    def __init__(self):
-        self.nodes = []
-        self.edges_dict = defaultdict(list)
-
-    def add_edge(self, source, target):
-        for node in [source, target]:
-            if node not in self.nodes:
-                self.nodes.append(node)
-        self.edges_dict[self.nodes.index(source)].append(self.nodes.index(target))
-
-    def get_topological_order(self):
-        exploring = [False]*len(self.nodes)
-        visited = [False]*len(self.nodes)
-        order = []
-        for source in range(len(self.nodes)):
-            if not visited[source]:
-                self.topological_sort(source, exploring, visited, order)
-        return [self.nodes[n] for n in order]
-
-    def topological_sort(self, source, exploring, visited, order):
-        exploring[source] = True
-        for target in self.edges_dict[source]:
-            if exploring[target]:
-                raise ValueError(f'Graph has cycles, found while checking {self.nodes[source]} --> '+self.nodes[target])
-            elif not visited[target]:
-                self.topological_sort(target, exploring, visited, order)
-        visited[source] = True
-        exploring[source] = False
-        order.insert(0, source)
-
-
 class ClassFromFunctionBase:
     wrapped_function: Callable
 
@@ -371,6 +338,9 @@ class Path:
     absolute path can be obtained without having to remember the working
     directory from when the object was created.
     """
+
+    file_scheme = re.compile('^file:///?')
+
     def __init__(
         self,
         path: Union[str, 'Path'],
@@ -404,8 +374,8 @@ class Path:
             path = path.rel_path  # type: ignore
         elif isinstance(path, str):
             abs_path = os.path.expanduser(path)
-            if re.match('^file:///?', abs_path):
-                abs_path = re.sub('^file:///?', '/', abs_path)
+            if self.file_scheme.match(abs_path):
+                abs_path = self.file_scheme.sub('' if os.name == 'nt' else '/', abs_path)
             if 'u' in mode and url_support and import_url_validator('Path')(abs_path):
                 is_url = True
             elif 's' in mode and fsspec_support and known_to_fsspec(abs_path):
@@ -591,8 +561,10 @@ class LoggerProperty:
     def logger(self, logger: Union[bool, str, dict, logging.Logger]):
         if logger is None:
             from .deprecated import deprecation_warning, logger_property_none_message
-            deprecation_warning(LoggerProperty, logger_property_none_message)
+            deprecation_warning((LoggerProperty.logger, None), logger_property_none_message)
             logger = False
+        if not logger and 'JSONARGPARSE_DEBUG' in os.environ:
+            logger = {'level': 'DEBUG'}
         self._logger = parse_logger(logger, type(self).__name__)
 
 
