@@ -3,7 +3,6 @@
 import json
 import os
 import pickle
-import platform
 import sys
 import unittest
 import warnings
@@ -40,8 +39,8 @@ from jsonargparse.optionals import (
     url_support,
 )
 from jsonargparse.typing import NotEmptyStr, Path_fc, Path_fr, PositiveFloat, PositiveInt
-from jsonargparse.util import CaptureParserException, capture_parser, DebugException
-from jsonargparse_tests.base import TempDirTestCase, responses, responses_activate
+from jsonargparse.util import CaptureParserException, capture_parser, DebugException, null_logger
+from jsonargparse_tests.base import is_posix, responses_activate, responses_available, TempDirTestCase
 
 
 def example_parser():
@@ -167,6 +166,24 @@ class ParsersTests(TempDirTestCase):
             self.assertFalse(parser.default_env)
 
 
+    def test_env_prefix(self):
+        parser = ArgumentParser(env_prefix=True, default_env=True, error_handler=None)
+        parser.add_argument("--test_arg", type=str, required=True, help="Test argument")
+        with self.assertRaises(ParserError):
+            with unittest.mock.patch.dict(os.environ, {'TEST_ARG': 'one'}):
+                parser.parse_args([])
+        prefix = os.path.splitext(parser.prog)[0].upper()
+        with unittest.mock.patch.dict(os.environ, {f'{prefix}_TEST_ARG': 'one'}):
+            cfg = parser.parse_args([])
+            self.assertEqual('one', cfg.test_arg)
+
+        parser = ArgumentParser(env_prefix=False, default_env=True)
+        parser.add_argument("--test_arg", type=str, required=True, help="Test argument")
+        with unittest.mock.patch.dict(os.environ, {'TEST_ARG': 'one'}):
+            cfg = parser.parse_args([])
+            self.assertEqual('one', cfg.test_arg)
+
+
     def test_parse_string(self):
         parser = example_parser()
 
@@ -375,23 +392,6 @@ class ArgumentFeaturesTests(unittest.TestCase):
 
 class AdvancedFeaturesTests(unittest.TestCase):
 
-    def test_link_arguments(self):
-        parser = ArgumentParser(error_handler=None)
-        parser.add_argument('--a.v1', default=2)
-        parser.add_argument('--a.v2', type=int, default=3)
-        parser.add_argument('--b.v2', type=int, default=4)
-        def a_prod(a):
-            return a['v1'] * a['v2']
-        parser.link_arguments('a', 'b.v2', a_prod)
-
-        cfg = parser.parse_args(['--a.v2=-5'])
-        self.assertEqual(cfg.b.v2, cfg.a.v1*cfg.a.v2)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--a.v1=x']))
-
-        dump = yaml.safe_load(parser.dump(cfg))
-        self.assertEqual(dump, {'a': {'v1': 2, 'v2': -5}})
-
-
     def test_subcommands(self):
         parser_a = ArgumentParser(error_handler=None)
         parser_a.add_argument('ap1')
@@ -553,7 +553,7 @@ class AdvancedFeaturesTests(unittest.TestCase):
         self.assertEqual(yaml.safe_load(out.getvalue()), {'o': 1})
 
 
-    @unittest.skipIf(not url_support or not responses, 'validators, requests and responses packages are required')
+    @unittest.skipIf(not (url_support and responses_available), 'validators, requests and responses packages are required')
     @responses_activate
     def test_urls(self):
         set_config_read_mode(urls_enabled=True)
@@ -600,6 +600,7 @@ class AdvancedFeaturesTests(unittest.TestCase):
             'jsonnet.yaml': jsonnet_body,
         }
 
+        import responses
         for name, body in urls.items():
             responses.add(responses.GET,
                           base_url+name,
@@ -931,7 +932,7 @@ class ConfigFilesTests(TempDirTestCase):
         outval = ' '.join(out.getvalue().split())
         self.assertIn('tried getting defaults considering default_config_files but failed', outval)
 
-        if os.name == 'posix' and platform.python_implementation() == 'CPython':
+        if is_posix:
             os.chmod(default_config_file, 0)
             self.assertEqual(parser.get_default('op1'), 'from default')
 
@@ -1084,7 +1085,7 @@ class OtherTests(unittest.TestCase):
 
     @unittest.mock.patch.dict(os.environ, {'JSONARGPARSE_DEBUG': ''})
     def test_debug_usage_and_exit_error_handler(self):
-        parser = ArgumentParser()
+        parser = ArgumentParser(logger=null_logger)
         parser.add_argument('--int', type=int)
         err = StringIO()
         with redirect_stderr(err), self.assertRaises(DebugException):
