@@ -55,6 +55,14 @@ class ClassC(ClassB):
         """
         super().__init__(**kargs)
 
+class ClassN:
+    def __init__(self, kn1: int = 1, **kargs):
+        """
+        Args:
+            kn1: help for kn1
+        """
+        self.kn2: int  # verify (though questionable stylistically) that value-less AnnAssign doesn't break resolution
+
 class Param:
     p1: int = 2
 
@@ -154,6 +162,64 @@ class ClassG:
             kmg3: help for kmg3
         """
 
+class ClassM1:
+    def __init__(self, km1: int = 1):
+        """
+        Args:
+            km1: help for km1
+        """
+
+class ClassM2(ClassM1):
+    def __init__(self, km2: int = 2, **kwargs):
+        """
+        Args:
+            km2: help for km2
+        """
+        super().__init__(**kwargs)
+
+class ClassM3(ClassM1):
+    def __init__(self, km3: int = 0, **kwargs):
+        """
+        Args:
+            km3: help for km3
+        """
+        super().__init__(**kwargs)
+
+class ClassP:
+    def __init__(self, kp1: int = 1, **kw):
+        """
+        Args:
+            kp1: help for kp1
+        """
+        self._kw = kw
+
+    @property
+    def data(self):
+        return function_no_args_no_kwargs(**self._kw)
+
+class ClassS1:
+    def __init__(self, ks1: int = 2, **kw):
+        """
+        Args:
+            ks1: help for ks1
+        """
+        self.ks1 = ks1
+
+    @classmethod
+    def classmethod_s(cls, **kwargs):
+        return cls(**kwargs)
+
+class ClassS2:
+    def __init__(self, **kwargs):
+        self.kwargs = kwargs
+
+    def run_classmethod_s(self):
+        return ClassS1.classmethod_s(**self.kwargs)
+
+class ClassM(ClassM2, ClassM3):
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+
 class ClassU1:
     def __init__(self, k1: int = 1, **ka):
         data = Namespace()
@@ -213,6 +279,20 @@ def function_make_class_b(*args, k1: str = '-', **kwargs):
     """
     return ClassB.make(*args, **kwargs)
 
+def function_pop_get_from_kwargs(kn1: int = 0, **kw):
+    """
+    Args:
+        kn1: help for kn1
+        kn2: help for kn2
+        kn3: help for kn3
+        kn4: help for kn4
+    """
+    k2 = kw.pop('k2', 2)
+    kn2 = kw.pop('kn2', 0.5)
+    kn3 = kw.get('kn3', {})
+    kn4 = kw.pop('kn4', [1])
+    return function_no_args_no_kwargs(**kw)
+
 def function_with_bug(**kws):
     return does_not_exist(**kws)  # pylint: disable=undefined-variable
 
@@ -221,6 +301,9 @@ def function_unsupported_component(**kwds):
     shuffle(select)
     getattr(calendar, f'{select[0]}Calendar')(**kwds)
 
+def function_module_class(**kwds):
+    return calendar.Calendar(**kwds)
+
 
 @contextmanager
 def source_unavailable():
@@ -234,7 +317,7 @@ def assert_params(self, params, expected):
     self.assertEqual(docs, [p.doc for p in params])
 
 
-logger = logging.getLogger('ast_analysis_tests')
+logger = logging.getLogger('test_parameter_resolvers')
 logger.level = logging.DEBUG
 
 
@@ -256,6 +339,11 @@ class GetClassParametersTests(unittest.TestCase):
         with source_unavailable():
             assert_params(self, get_params(ClassC), ['kc1', 'pkb1', 'kb1', 'kb2', 'ka1', 'ka2'])
 
+    def test_get_params_class_with_valueless_init_ann(self):
+        assert_params(self, get_params(ClassN), ['kn1'])
+        with source_unavailable():
+            assert_params(self, get_params(ClassN), ['kn1'])
+
     def test_get_params_class_with_inheritance_parent_without_init(self):
         params = get_params(ClassD)
         assert_params(self, params, ['kd1', 'ka1', 'ka2'])
@@ -275,7 +363,17 @@ class GetClassParametersTests(unittest.TestCase):
         with source_unavailable():
             assert_params(self, get_params(ClassG), ['func'])
 
-    def test_class_from_function(self):
+    def test_get_params_method_resolution_order(self):
+        assert_params(self, get_params(ClassM), ['km2', 'km3', 'km1'])
+        with source_unavailable():
+            assert_params(self, get_params(ClassM), ['km2', 'km3', 'km1'])
+
+    def test_get_params_kwargs_use_in_property(self):
+        assert_params(self, get_params(ClassP), ['kp1', 'pk1', 'k2'])
+        with source_unavailable():
+            assert_params(self, get_params(ClassP), ['kp1'])
+
+    def test_get_params_class_from_function(self):
         class_a = class_from_function(function_return_class_c)
         params = get_params(class_a)
         assert_params(self, params, ['pk1', 'k2', 'pkb1', 'kb2', 'ka1'])
@@ -311,6 +409,12 @@ class GetMethodParametersTests(unittest.TestCase):
         with source_unavailable():
             assert_params(self, get_params(ClassB.make), ['pkcm1', 'kcm1'])
 
+    def test_get_params_classmethod_instantiate_from_cls(self):
+        assert_params(self, get_params(ClassS1, 'classmethod_s'), ['ks1'])
+        assert_params(self, get_params(ClassS2), ['ks1'])
+        with source_unavailable():
+            assert_params(self, get_params(ClassS1, 'classmethod_s'), [])
+
 
 class GetFunctionParametersTests(unittest.TestCase):
 
@@ -335,18 +439,31 @@ class GetFunctionParametersTests(unittest.TestCase):
         with source_unavailable():
             assert_params(self, get_params(function_make_class_b), ['k1'])
 
+    def test_get_params_function_pop_get_from_kwargs(self):
+        with self.assertLogs(logger, level='DEBUG') as log:
+            params = get_params(function_pop_get_from_kwargs, logger=logger)
+            assert_params(self, params, ['kn1', 'kn2', 'kn3', 'kn4', 'pk1', 'k2'])
+            self.assertIs(params[-1].annotation, int)
+            self.assertIn('unsupported kwargs pop/get default', log.output[0])
+        with source_unavailable():
+            assert_params(self, get_params(function_pop_get_from_kwargs), ['kn1'])
+
+    def test_get_params_function_module_class(self):
+        params = get_params(function_module_class)
+        self.assertEqual(['firstweekday'], [p.name for p in params])
+
 
 class OtherTests(unittest.TestCase):
 
     def test_unsupported_component(self):
         with self.assertLogs(logger, level='DEBUG') as log:
             self.assertEqual([], get_params(function_unsupported_component, logger=logger))
-            self.assertIn('Component not supported', log.output[0])
+            self.assertIn('not supported', log.output[0])
 
     def test_unsupported_type_of_assign(self):
         with self.assertLogs(logger, level='DEBUG') as log:
             get_params(ClassU1, logger=logger)
-            self.assertIn('Unsupported type of assign', log.output[0])
+            self.assertIn('unsupported type of assign', log.output[0])
 
     def test_unsupported_kwarg_as_keyword(self):
         with self.assertLogs(logger, level='DEBUG') as log:
@@ -361,18 +478,23 @@ class OtherTests(unittest.TestCase):
     def test_unsupported_self_attr_not_found_in_members(self):
         with self.assertLogs(logger, level='DEBUG') as log:
             get_params(ClassU4, logger=logger)
-            self.assertIn('Did not find use of self._ka in members of', log.output[0])
+            self.assertIn('did not find use of self._ka in members of', log.output[0])
 
     def test_unsupported_kwarg_attr_as_keyword(self):
         with self.assertLogs(logger, level='DEBUG') as log:
             get_params(ClassU5, logger=logger)
             self.assertIn('kwargs attribute given as keyword parameter not supported', log.output[0])
 
+    def test_get_params_non_existent_call(self):
+        with self.assertLogs(logger, level='DEBUG') as log:
+            self.assertEqual([], get_params(function_with_bug, logger=logger))
+            self.assertIn('does_not_exist', log.output[0])
+            self.assertIn('Problems with AST resolving', log.output[0])
+
     def test_get_params_failures(self):
         self.assertRaises(ValueError, lambda: get_params('invalid'))
         self.assertRaises(ValueError, lambda: get_params(Param, 'p1'))
         self.assertRaises(AttributeError, lambda: get_params(Param, 'p2'))
-        self.assertRaises(Exception, lambda: get_params(function_with_bug))
 
 
 if __name__ == '__main__':
