@@ -416,6 +416,23 @@ class SignaturesTests(unittest.TestCase):
             self.assertIn("discarding init_args: {'pa': 'A', 'pc': 'X'}", str(w[0].message))
 
 
+    def test_class_path_override_with_mixed_type(self):
+        class MyCalendar(Calendar):
+            def __init__(self, *args, param: int = 0, **kwargs):
+                super().__init__(*args, **kwargs)
+
+        class Main:
+            def __init__(self, cal: Union[Calendar, bool] = lazy_instance(MyCalendar, param=1)):
+                self.cal = cal
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_class_arguments(Main, 'main')
+
+        with warnings.catch_warnings(record=True) as w:
+            parser.parse_args(['--main.cal=Calendar'])
+            self.assertIn("discarding init_args: {'param': 1}", str(w[0].message))
+
+
     def test_add_subclass_init_args_without_class_path(self):
         parser = ArgumentParser(error_handler=None)
         parser.add_subclass_arguments(Calendar, 'cal1')
@@ -896,322 +913,6 @@ class SignaturesTests(unittest.TestCase):
             self.assertEqual(out.getvalue(), expected)
 
 
-    def test_link_arguments(self):
-
-        class ClassA:
-            def __init__(self, v1: int = 2, v2: int = 3):
-                pass
-
-        class ClassB:
-            def __init__(self, v1: int = -1, v2: int = 4, v3: int = 2):
-                """ClassB title
-                Args:
-                    v1: b v1 help
-                """
-
-        parser = ArgumentParser(error_handler=None)
-        parser.add_class_arguments(ClassA, 'a')
-        parser.add_class_arguments(ClassB, 'b')
-        parser.link_arguments('a.v2', 'b.v1')
-        def add(*args):
-            return sum(args)
-        parser.link_arguments(('a.v1', 'a.v2'), 'b.v2', add)
-
-        cfg = parser.parse_args([])
-        self.assertEqual(cfg.b.v1, cfg.a.v2)
-        self.assertEqual(cfg.b.v2, cfg.a.v1+cfg.a.v2)
-        cfg = parser.parse_args(['--a.v1=11', '--a.v2=7'])
-        self.assertEqual(7, cfg.b.v1)
-        self.assertEqual(11+7, cfg.b.v2)
-        self.assertEqual(Namespace(), parser.parse_args([], defaults=False))
-
-        dump = yaml.safe_load(parser.dump(cfg))
-        self.assertEqual(dump, {'a': {'v1': 11, 'v2': 7}, 'b': {'v3': 2}})
-
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--b.v1=5']))
-        self.assertRaises(ValueError, lambda: parser.link_arguments('a.v2', 'b.v1'))
-        self.assertRaises(ValueError, lambda: parser.link_arguments('x', 'b.v2'))
-        self.assertRaises(ValueError, lambda: parser.link_arguments('a.v1', 'x'))
-        self.assertRaises(ValueError, lambda: parser.link_arguments(('a.v1', 'a.v2'), 'b.v3'))
-        self.assertRaises(ValueError, lambda: parser.link_arguments('a.v1', 'b.v2', apply_on='bad'))
-
-        help_str = StringIO()
-        parser.print_help(help_str)
-        self.assertIn('Linked arguments', help_str.getvalue())
-        self.assertIn('a.v2 --> b.v1', help_str.getvalue())
-        self.assertIn('add(a.v1, a.v2) --> b.v2', help_str.getvalue())
-        if docstring_parser_support:
-            self.assertIn('b v1 help', help_str.getvalue())
-
-
-    def test_link_arguments_subclasses(self):
-        class ClassA:
-            def __init__(
-                self,
-                v1: Union[int, str] = 1,
-                v2: Union[int, str] = 2,
-            ):
-                pass
-
-        parser = ArgumentParser(error_handler=None)
-        parser.add_subclass_arguments(ClassA, 'a')
-        parser.add_subclass_arguments(Calendar, 'c')
-
-        def add(v1, v2):
-            return v1 + v2
-        parser.link_arguments(('a.init_args.v1', 'a.init_args.v2'), 'c.init_args.firstweekday', add)
-
-        with mock_module(ClassA) as module:
-            a_value = {
-                'class_path': f'{module}.ClassA',
-                'init_args': {'v2': 3},
-            }
-
-            cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
-            self.assertEqual(cfg.c.init_args.firstweekday, 4)
-            self.assertEqual(cfg.c.init_args.firstweekday, cfg.a.init_args.v1+cfg.a.init_args.v2)
-
-            cfg_init = parser.instantiate_classes(cfg)
-            self.assertIsInstance(cfg_init.a, ClassA)
-            self.assertIsInstance(cfg_init.c, Calendar)
-
-            dump = yaml.safe_load(parser.dump(cfg))
-            a_value['init_args']['v1'] = 1
-            self.assertEqual(dump, {'a': a_value, 'c': {'class_path': 'calendar.Calendar'}})
-
-            self.assertRaises(ValueError, lambda: parser.link_arguments('a.init_args.v1', 'c'))
-
-            a_value['init_args'] = {'v1': 'a', 'v2': 'b'}
-            with self.assertRaises(ParserError):
-                parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
-
-
-    def test_link_arguments_subclasses_with_instantiate_false(self):
-        class ClassA:
-            def __init__(
-                self,
-                v: Union[int, str] = 1,
-                c: Optional[Calendar] = None,
-            ):
-                self.c = c
-
-        parser = ArgumentParser(error_handler=None)
-        parser.add_subclass_arguments(ClassA, 'a')
-        parser.add_subclass_arguments(Calendar, 'c', instantiate=False)
-        parser.link_arguments('c', 'a.init_args.c')
-
-        with mock_module(ClassA) as module:
-            a_value = {'class_path': f'{module}.ClassA'}
-            c_value = {
-                'class_path': 'calendar.Calendar',
-                'init_args': {
-                    'firstweekday': 3,
-                },
-            }
-
-            cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c='+json.dumps(c_value)])
-            self.assertEqual(cfg.c.as_dict(), {'class_path': 'calendar.Calendar', 'init_args': {'firstweekday': 3}})
-            self.assertEqual(cfg.c, cfg.a.init_args.c)
-
-            cfg_init = parser.instantiate_classes(cfg)
-            self.assertIsInstance(cfg_init.c, Namespace)
-            self.assertIsInstance(cfg_init.a, ClassA)
-            self.assertIsInstance(cfg_init.a.c, Calendar)
-            self.assertEqual(cfg_init.a.c.firstweekday, 3)
-
-            dump = yaml.safe_load(parser.dump(cfg))
-            self.assertNotIn('c', dump['a']['init_args'])
-
-
-    def test_link_arguments_subclass_as_dict(self):
-        class ClassA:
-            def __init__(
-                self,
-                a1: dict,
-                a2: Optional[dict] = None,
-                a3: Any = None,
-            ):
-                pass
-
-        def return_dict(value: dict):
-            return value
-
-        parser = ArgumentParser(error_handler=None)
-        parser.add_subclass_arguments(ClassA, 'a')
-        parser.add_subclass_arguments(Calendar, 'c')
-        parser.link_arguments('c', 'a.init_args.a1', compute_fn=return_dict)
-        parser.link_arguments('c', 'a.init_args.a2')
-        parser.link_arguments('c', 'a.init_args.a3')
-
-        with mock_module(ClassA) as module:
-            a_value = {'class_path': f'{module}.ClassA'}
-            c_value = {
-                'class_path': 'calendar.Calendar',
-                'init_args': {
-                    'firstweekday': 3,
-                },
-            }
-
-            cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c='+json.dumps(c_value)])
-            self.assertEqual(cfg.a.init_args.a1, c_value)
-            self.assertEqual(cfg.a.init_args.a2, c_value)
-
-
-    def test_link_arguments_subcommand(self):
-        class Foo:
-            def __init__(self, a: int):
-                self.a = a
-
-        parser = ArgumentParser()
-        subparser = ArgumentParser()
-
-        subcommands = parser.add_subcommands()
-        subparser.add_class_arguments(Foo, nested_key='foo')
-        subparser.add_argument('--b', type=int)
-        subparser.link_arguments('b', 'foo.a')
-        subcommands.add_subcommand('cmd', subparser)
-
-        cfg = parser.parse_args(['cmd', '--b=2'])
-        self.assertEqual(cfg['cmd']['foo'].as_dict(), {'a': 2})
-
-        cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['cmd']['foo'], Foo)
-
-
-    def test_link_arguments_apply_on_instantiate(self):
-
-        class ClassA:
-            def __init__(self, a1: int, a2: float = 2.3):
-                self.a1 = a1
-                self.a2 = a2
-
-        class ClassB:
-            def __init__(self, b1: float = 4.5, b2: int = 6, b3: str = '7'):
-                self.b1 = b1
-                self.b2 = b2
-                self.b3 = b3
-
-        class ClassC:
-            def __init__(self, c1: int = 7, c2: str = '8'):
-                self.c1 = c1
-                self.c2 = c2
-
-        def make_parser_1():
-            parser = ArgumentParser(error_handler=None)
-            parser.add_class_arguments(ClassA, 'a')
-            parser.add_class_arguments(ClassB, 'b')
-            parser.add_class_arguments(ClassC, 'c')
-            parser.add_argument('--d', type=int, default=-1)
-            parser.link_arguments('b.b2', 'a.a1', apply_on='instantiate')
-            parser.link_arguments('c.c1', 'b.b1', apply_on='instantiate')
-            return parser
-
-        def get_b2(obj_b):
-            return obj_b.b2
-
-        def make_parser_2():
-            parser = ArgumentParser(error_handler=None)
-            parser.add_subclass_arguments(ClassA, 'a')
-            parser.add_subclass_arguments(ClassB, 'b')
-            parser.link_arguments('b', 'a.init_args.a1', get_b2, apply_on='instantiate')
-            return parser
-
-        # Link object attribute
-        parser = make_parser_1()
-        parser.link_arguments('c.c2', 'b.b3', compute_fn=lambda v: f'"{v}"', apply_on='instantiate')
-        cfg = parser.parse_args([])
-        cfg = parser.instantiate_classes(cfg)
-        self.assertEqual(cfg.a.a1, 6)
-        self.assertEqual(cfg.b.b3, '"8"')
-
-        # Link all group arguments
-        parser = make_parser_1()
-        parser.link_arguments('b.b1', 'a.a2', apply_on='instantiate')
-        cfg = parser.parse_args([])
-        cfg = parser.instantiate_classes(cfg)
-        self.assertEqual(cfg['a'].a1, 6)
-        self.assertEqual(cfg['a'].a2, 7)
-
-        # Links with cycle
-        parser = make_parser_1()
-        with self.assertRaises(ValueError):
-            parser.link_arguments('a.a2', 'b.b3', apply_on='instantiate')
-        parser = make_parser_1()
-        with self.assertRaises(ValueError):
-            parser.link_arguments('a.a2', 'c.c2', apply_on='instantiate')
-
-        # Not subclass action or a class group
-        parser = make_parser_1()
-        with self.assertRaises(ValueError):
-            parser.link_arguments('d', 'c.c2', apply_on='instantiate')
-
-        # Link subclass and compute function
-        with mock_module(ClassA, ClassB) as module:
-            parser = make_parser_2()
-            cfg = parser.parse_args([
-                f'--a={module}.ClassA',
-                f'--b={module}.ClassB',
-            ])
-            cfg = parser.instantiate_classes(cfg)
-            self.assertEqual(cfg['a'].a1, 6)
-
-        # Unsupported multi-source
-        parser = make_parser_2()
-        with self.assertRaises(ValueError):
-            parser.link_arguments(('b.b2', 'c.c1'), 'b.b3', compute_fn=lambda x, y: x, apply_on='instantiate')
-
-        # Source must be subclass action or class group
-        parser = make_parser_2()
-        with self.assertRaises(ValueError):
-            parser.link_arguments('a.b.c', 'b.b3', apply_on='instantiate')
-
-        # Object link without compute function
-        parser = make_parser_2()
-        with self.assertRaises(ValueError):
-            parser.link_arguments('b', 'a.init_args.a2', apply_on='instantiate')
-
-        # Unsupported deeper levels
-        parser = ArgumentParser(error_handler=None)
-        parser.add_class_arguments(ClassA, 'g.a')
-        parser.add_class_arguments(ClassB, 'g.b')
-        with self.assertRaises(ValueError):
-            parser.link_arguments('g.b.b2', 'g.a.a1', apply_on='instantiate')
-
-
-    def test_link_arguments_subclass_missing_param_issue_129(self):
-        with suppress_stderr():
-            class ClassA:
-                def __init__(self, a1: int = 1):
-                    self.a1 = a1
-
-            class ClassB:
-                def __init__(self, b1: int = 2):
-                    self.b1 = b1
-
-            parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
-            with mock_module(ClassA, ClassB) as module, self.assertLogs(logger=parser.logger, level='DEBUG') as log:
-                parser.add_subclass_arguments(ClassA, 'a', default=lazy_instance(ClassA))
-                parser.add_subclass_arguments(ClassB, 'b', default=lazy_instance(ClassB))
-                parser.link_arguments('a.init_args.a2', 'b.init_args.b1', apply_on='parse')
-                parser.link_arguments('a.init_args.a1', 'b.init_args.b2', apply_on='parse')
-
-                parser.parse_args([f'--a={module}.ClassA', f'--b={module}.ClassB'])
-                self.assertTrue(any('a.init_args.a2 --> b.init_args.b1 ignored since source' in x for x in log.output))
-                self.assertTrue(any('a.init_args.a1 --> b.init_args.b2 ignored since target' in x for x in log.output))
-
-            parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
-            with mock_module(ClassA, ClassB) as module, self.assertLogs(logger=parser.logger, level='DEBUG') as log:
-                parser.add_subclass_arguments(ClassA, 'a', default=lazy_instance(ClassA))
-                parser.add_subclass_arguments(ClassB, 'b', default=lazy_instance(ClassB))
-                parser.link_arguments('a.init_args.a2', 'b.init_args.b1', apply_on='instantiate')
-                parser.link_arguments('a.init_args.a1', 'b.init_args.b2', apply_on='instantiate')
-
-                cfg = parser.parse_args([f'--a={module}.ClassA', f'--b={module}.ClassB'])
-                parser.instantiate_classes(cfg)
-                self.assertTrue(any('a.init_args.a2 --> b.init_args.b1 ignored since source' in x for x in log.output))
-                self.assertTrue(any('a.init_args.a1 --> b.init_args.b2 ignored since target' in x for x in log.output))
-
-
     def test_class_from_function(self):
 
         def get_calendar() -> Calendar:
